/*
	RE3
	PS1
	Game

	Copyright (C) 2007-2010	Patrice Mandin

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <stdlib.h>

#include <SDL.h>

#include "filesystem.h"
#include "state.h"
#include "re3_ps1_game.h"
#include "background_bss.h"
#include "parameters.h"
#include "log.h"
#include "room_rdt2.h"
#include "render.h"

/*--- Defines ---*/

#define CHUNK_SIZE 65536

/*--- Types ---*/

typedef struct {
	Uint32 length;
	Uint32 count;
} ard_header_t;

typedef struct {
	Uint32 length;
	Uint32 unknown;
} ard_object_t;

typedef struct {
	Uint32 start;	/* Start sector */
	Uint16 count;	/* Number of sectors */
	const char *filename;	/* File stored there */
} re3_cdraw_t;

/*--- Constant ---*/

static const char *re3ps1game_bg = "cd_data/stage%d/r%d%02x.bss";
static const char *re3ps1game_room = "cd_data/stage%d/r%d%02x.ard";
static const char *re3ps1game_font = "cd_data/etc/sele_ob%c.tim";

static const char *re3ps1game_movies[] = {
	"cd_data/zmovie/enda.str",
	"cd_data/zmovie/endb.str",
	"cd_data/zmovie/ins01.str",
	"cd_data/zmovie/ins02.str",
	"cd_data/zmovie/ins03.str",
	"cd_data/zmovie/ins04.str",
	"cd_data/zmovie/ins05.str",
	"cd_data/zmovie/ins06.str",
	"cd_data/zmovie/ins07.str",
	"cd_data/zmovie/ins08.str",
	"cd_data/zmovie/ins09.str",
	"cd_data/zmovie/opn.str",
	"cd_data/zmovie/roopne.str",
	NULL
};

static const re3_cdraw_t re3_sles_02530[]={
	{126,76,"capcom.tim"},
	{202,82,"continue.tim"},
	{284,32,"core00f.tim"},
	{316,25,"died00f.tim"},
	{341,86,"eidos.tim"},
	{427,76,"epi_sel.tim"},
	{503,17,"epi_none.tim"},
	{520,17,"insta_i.tim"},
	{537,12,"insta_t0.tim"},
	{549,12,"insta_t1.tim"},
	{561,12,"insta_t2.tim"},
	{573,12,"insta_t3.tim"},
	{585,12,"insta_t4.tim"},
	{597,12,"insta_t5.tim"},
	{609,12,"insta_t6.tim"},
	{621,12,"insta_t7.tim"},
	{633,12,"insta_t8.tim"},
	{645,17,"dario_i.tim"},
	{662,12,"dario_t0.tim"},
	{674,12,"dario_t1.tim"},
	{686,12,"dario_t2.tim"},
	{698,12,"dario_t3.tim"},
	{710,12,"dario_t4.tim"},
	{722,17,"dmerc_i.tim"},
	{739,12,"dmerc_t0.tim"},
	{751,12,"dmerc_t1.tim"},
	{763,12,"dmerc_t2.tim"},
	{775,12,"dmerc_t3.tim"},
	{787,12,"dmerc_t4.tim"},
	{799,12,"dmerc_t5.tim"},
	{811,12,"dmerc_t6.tim"},
	{823,17,"fax1_i.tim"},
	{840,12,"fax1_t0.tim"},
	{852,12,"fax1_t1.tim"},
	{864,17,"marvin_i.tim"},
	{881,12,"marvin_t0.tim"},
	{893,12,"marvin_t1.tim"},
	{905,12,"marvin_t2.tim"},
	{917,12,"marvin_t3.tim"},
	{929,12,"marvin_t4.tim"},
	{941,17,"david_i.tim"},
	{958,12,"david_t0.tim"},
	{970,12,"david_t1.tim"},
	{982,12,"david_t2.tim"},
	{994,12,"david_t3.tim"},
	{1006,12,"david_t4.tim"},
	{1018,17,"town_i.tim"},
	{1035,12,"town_t0.tim"},
	{1047,12,"town_t1.tim"},
	{1059,12,"town_t2.tim"},
	{1071,12,"town_t3.tim"},
	{1083,12,"town_t4.tim"},
	{1095,17,"report_i.tim"},
	{1112,12,"report_t0.tim"},
	{1124,12,"report_t1.tim"},
	{1136,12,"report_t2.tim"},
	{1148,12,"report_t3.tim"},
	{1160,12,"report_t4.tim"},
	{1172,12,"report_t5.tim"},
	{1184,17,"inst_i.tim"},
	{1201,12,"inst_t0.tim"},
	{1213,12,"inst_t1.tim"},
	{1225,12,"inst_t2.tim"},
	{1237,12,"inst_t3.tim"},
	{1249,12,"inst_t4.tim"},
	{1261,17,"nmerc_i.tim"},
	{1278,12,"nmerc_t0.tim"},
	{1290,12,"nmerc_t1.tim"},
	{1302,12,"nmerc_t2.tim"},
	{1314,12,"nmerc_t3.tim"},
	{1326,12,"nmerc_t4.tim"},
	{1338,12,"nmerc_t5.tim"},
	{1350,12,"nmerc_t6.tim"},
	{1362,17,"postcard_i.tim"},
	{1379,12,"postcard_t0.tim"},
	{1391,12,"postcard_t1.tim"},
	{1403,12,"postcard_t2.tim"},
	{1415,17,"super_i.tim"},
	{1432,12,"super_t0.tim"},
	{1444,12,"super_t1.tim"},
	{1456,12,"super_t2.tim"},
	{1468,12,"super_t3.tim"},
	{1480,12,"super_t4.tim"},
	{1492,12,"super_t5.tim"},
	{1504,17,"order_i.tim"},
	{1521,12,"order_t0.tim"},
	{1533,12,"order_t1.tim"},
	{1545,12,"order_t2.tim"},
	{1557,17,"director_i.tim"},
	{1574,12,"director_t0.tim"},
	{1586,12,"director_t1.tim"},
	{1598,12,"director_t2.tim"},
	{1610,12,"director_t3.tim"},
	{1622,12,"director_t4.tim"},
	{1634,12,"director_t5.tim"},
	{1646,12,"director_t6.tim"},
	{1658,17,"manager1_i.tim"},
	{1675,12,"manager1_t0.tim"},
	{1687,12,"manager1_t1.tim"},
	{1699,12,"manager1_t2.tim"},
	{1711,12,"manager1_t3.tim"},
	{1723,12,"manager1_t4.tim"},
	{1735,12,"manager1_t5.tim"},
	{1747,12,"manager1_t6.tim"},
	{1759,12,"manager1_t7.tim"},
	{1771,17,"security_i.tim"},
	{1788,12,"security_t0.tim"},
	{1800,12,"security_t1.tim"},
	{1812,12,"security_t2.tim"},
	{1824,12,"security_t3.tim"},
	{1836,17,"meca_i.tim"},
	{1853,12,"meca_t0.tim"},
	{1865,12,"meca_t1.tim"},
	{1877,12,"meca_t2.tim"},
	{1889,12,"meca_t3.tim"},
	{1901,12,"meca_t4.tim"},
	{1913,17,"kendo_i.tim"},
	{1930,12,"kendo_t0.tim"},
	{1942,12,"kendo_t1.tim"},
	{1954,12,"kendo_t2.tim"},
	{1966,12,"kendo_t3.tim"},
	{1978,12,"kendo_t4.tim"},
	{1990,12,"kendo_t5.tim"},
	{2002,17,"manager2_i.tim"},
	{2019,12,"manager2_t0.tim"},
	{2031,12,"manager2_t1.tim"},
	{2043,12,"manager2_t2.tim"},
	{2055,12,"manager2_t3.tim"},
	{2067,12,"manager2_t4.tim"},
	{2079,12,"manager2_t5.tim"},
	{2091,17,"medic_i.tim"},
	{2108,12,"medic_t0.tim"},
	{2120,12,"medic_t1.tim"},
	{2132,12,"medic_t2.tim"},
	{2144,12,"medic_t3.tim"},
	{2156,17,"fax2_i.tim"},
	{2173,12,"fax2_t0.tim"},
	{2185,12,"fax2_t1.tim"},
	{2197,17,"incinerator_i.tim"},
	{2214,12,"incinerator_t0.tim"},
	{2226,12,"incinerator_t1.tim"},
	{2238,12,"incinerator_t2.tim"},
	{2250,12,"incinerator_t3.tim"},
	{2262,17,"photoa_i.tim"},
	{2279,12,"photoa_t0.tim"},
	{2291,12,"photoa_t1.tim"},
	{2303,12,"photoa_t2.tim"},
	{2315,12,"photoa_t3.tim"},
	{2327,17,"photob_i.tim"},
	{2344,12,"photob_t0.tim"},
	{2356,12,"photob_t1.tim"},
	{2368,12,"photob_t2.tim"},
	{2380,12,"photob_t3.tim"},
	{2392,17,"photoc_i.tim"},
	{2409,12,"photoc_t0.tim"},
	{2421,12,"photoc_t1.tim"},
	{2433,12,"photoc_t2.tim"},
	{2445,17,"photoe_i.tim"},
	{2462,12,"photoe_t0.tim"},
	{2474,12,"photoe_t1.tim"},
	{2486,12,"photoe_t2.tim"},
	{2498,17,"photod_i.tim"},
	{2515,12,"photod_t0.tim"},
	{2527,12,"photod_t1.tim"},
	{2539,12,"photod_t2.tim"},
	{2551,12,"photod_t3.tim"},
	{2563,17,"belfry_i.tim"},
	{2580,12,"belfry_t0.tim"},
	{2592,12,"belfry_t1.tim"},
	{2604,12,"belfry_t2.tim"},
	{2616,12,"belfry_t3.tim"},
	{2628,17,"instb_i.tim"},
	{2645,12,"instb_t0.tim"},
	{2657,12,"instb_t1.tim"},
	{2669,12,"instb_t2.tim"},
	{2681,12,"instb_t3.tim"},
	{2693,12,"instb_t4.tim"},
	{2705,12,"instb_t5.tim"},
	{2717,12,"instb_t6.tim"},
	{2729,12,"instb_t7.tim"},
	{2741,17,"secret_i.tim"},
	{2758,12,"secret_t0.tim"},
	{2770,12,"secret_t1.tim"},
	{2782,12,"secret_t2.tim"},
	{2794,17,"djill_i.tim"},
	{2811,12,"djill_t0.tim"},
	{2823,12,"djill_t1.tim"},
	{2835,12,"djill_t2.tim"},
	{2847,12,"djill_t3.tim"},
	{2859,12,"djill_t4.tim"},
	{2871,12,"djill_t5.tim"},
	{2883,12,"djill_t6.tim"},
	{2895,12,"djill_t7.tim"},
	{2907,17,"fileif.tim"},
	{2924,17,"filei.tim"},
	{2941,65,"fontst0f.tim"},
	{3006,65,"fontst1f.tim"},
	{3071,65,"fontst2f.tim"},
	{3136,65,"fontst3f.tim"},
	{3201,65,"fontst4f.tim"},
	{3266,65,"fontst5f.tim"},
	{3331,65,"fontst6f.tim"},
	{3396,65,"fontst7f.tim"},
	{3461,44,"nemesis_icon.tim"},
	{3505,5,"colt1.tim"},
	{3510,5,"knife.tim"},
	{3515,5,"colt2.tim"},
	{3520,5,"colt3.tim"},
	{3525,5,"shotgun1.tim"},
	{3530,5,"magnum.tim"},
	{3535,5,"grenade_launcher.tim"},
	/*{3540,5,"grenade_launcher.tim"},*/ /*copy*/
	/*{3545,5,"grenade_launcher.tim"},*/ /*copy*/
	/*{3550,5,"grenade_launcher.tim"},*/ /*copy*/
	{3555,5,"rocket_launcher.tim"},
	{3560,5,"gattling.tim"},
	{3565,5,"supercolt.tim"},
	{3570,5,"lasercolt.tim"},
	{3575,5,"m16.tim"},
	/*{3580,5,"m16.tim"},*/ /*copy*/
	{3585,5,"shotgun2.tim"},
	/*{3590,5,"colt2.tim"},*/ /*copy*/
	/*{3595,5,"colt3.tim"},*/ /*copy*/
	/*{3600,5,"shotgun.tim"},*/ /*copy*/
	/*{3605,5,"supercolt.tim"},*/ /*copy*/
	{3610,5,"ammo_colt.tim"},
	{3615,5,"ammo_magnum.tim"},
	{3620,5,"ammo_shotgun.tim"},
	{3625,5,"ammo_grenade.tim"},
	{3630,5,"ammo_grenade_fire.tim"},
	{3635,5,"ammo_grenade_acid.tim"},
	{3640,5,"ammo_grenade_blue.tim"},
	{3645,5,"ammo_elec.tim"},
	{3650,5,"ammo_m16.tim"},
	{3655,5,"ammo_supercolt.tim"},
	{3660,5,"ammo_shotgun_blue.tim"},
	{3665,5,"spray.tim"},
	{3670,5,"herb_g.tim"},
	{3675,5,"herb_b.tim"},
	{3680,5,"herb_r.tim"},
	{3685,5,"herb_gg.tim"},
	{3690,5,"herb_gb.tim"},
	{3695,5,"herb_gr.tim"},
	{3700,5,"herb_ggg.tim"},
	{3705,5,"herb_ggb.tim"},
	{3710,5,"herb_gbr.tim"},
	{3715,5,"spray_box.tim"},
	{3720,5,"lever.tim"},
	{3725,5,"medal_r.tim"},
	{3730,5,"medal_b.tim"},
	{3735,5,"medal_y.tim"},
	{3740,5,"police_card1.tim"},
	{3745,5,"oil.tim"},
	{3750,5,"battery.tim"},
	{3755,5,"crowbar.tim"},
	{3760,5,"elec_cord.tim"},
	{3765,5,"xxx.tim"},
	{3770,5,"tuyau_arrose.tim"},
	{3775,5,"bottle.tim"},
	{3780,5,"etui.tim"},
	{3785,5,"police_card2.tim"},
	{3790,5,"gas_empty.tim"},
	{3795,5,"gas_filled.tim"},
	{3800,5,"chain.tim"},
	{3805,5,"spanner.tim"},
	{3810,5,"pipe.tim"},
	{3815,5,"plug.tim"},
	/*{3820,5,"tuyau_arrose.tim"},*/ /*copy*/
	{3825,5,"dictaphone.tim"},
	{3830,5,"lighter_gas.tim"},
	{3835,5,"lighter_closed.tim"},
	{3840,5,"lighter_opened.tim"},
	{3845,5,"gem_g.tim"},
	{3850,5,"gem_b.tim"},
	{3855,5,"ovalgem_y.tim"},
	{3860,5,"ovalgem_o.tim"},
	{3865,5,"ovalgem_w.tim"},
	{3870,5,"remote_opened.tim"},
	{3875,5,"remote_closed.tim"},
	{3880,5,"remote_battery.tim"},
	{3885,5,"gear_y.tim"},
	{3890,5,"gear_w.tim"},
	{3895,5,"gear_yw.tim"},
	{3900,5,"book.tim"},
	{3905,5,"sundial.tim"},
	{3910,5,"serum_r.tim"},
	{3915,5,"serum_b.tim"},
	/*{3920,5,"colt.tim"},*/ /*copy*/
	/*{3925,5,"colt.tim"},*/ /*copy*/
	{3930,5,"serum_p1.tim"},
	/*{3935,5,"colt.tim"},*/ /*copy*/
	/*{3940,5,"colt.tim"},*/ /*copy*/
	{3945,5,"storage.tim"},
	{3950,5,"colt_pieces1.tim"},
	{3955,5,"colt_pieces2.tim"},
	{3960,5,"shotgun_pieces1.tim"},
	{3965,5,"shotgun_pieces2.tim"},
	/*{3970,5,"colt.tim"},*/ /*copy*/
	{3975,5,"key_gem.tim"},
	{3980,5,"lever2.tim"},
	{3985,5,"umbrella_card.tim"},
	{3990,5,"ammo_r.tim"},
	{3995,5,"ammo_y.tim"},
	{4000,5,"ammo_b.tim"},
	{4005,5,"ammo_rr.tim"},
	{4010,5,"ammo_yy.tim"},
	{4015,5,"ammo_rb.tim"},
	{4020,5,"ammo_yb.tim"},
	{4025,5,"ammo_bb.tim"},
	{4030,5,"ammo_rrr.tim"},
	{4035,5,"ammo_rry.tim"},
	{4040,5,"ammo_ryy.tim"},
	{4045,5,"ammo_yyy.tim"},
	{4050,5,"ammo_bbb.tim"},
	{4055,5,"ammo_inf.tim"},
	{4060,5,"water.tim"},
	{4065,5,"modisc.tim"},
	{4070,5,"key_spade.tim"},
	{4075,5,"paperclip.tim"},
	{4080,5,"key_xxx.tim"},
	{4085,5,"key_paper_y.tim"},
	{4090,5,"key_xxx.tim"},
	{4095,5,"keychain.tim"},
	{4100,5,"key_hole1.tim"},
	{4105,5,"key_hole2.tim"},
	{4110,5,"key_hole.tim"},
	/*{4115,5,"colt.tim"},*/ /*copy */
	{4120,5,"key_wood.tim"},
	{4125,5,"key_xxx.tim"},
	{4130,5,"key_small.tim"},
	{4135,5,"key_card1.tim"},
	{4140,5,"key_card2.tim"},
	{4145,5,"key_gem.tim"},
	{4150,5,"ink_ribbon.tim"},
	{4155,5,"ammo_build.tim"},
	/*{4160,5,"insta_i.tim"},*/ /*copy*/
	/*{4165,5,"instb_i.tim"},*/ /*copy*/
	{4170,5,"syrup.tim"},
	{4309,76,"jillbgf.tim"},
	{4385,231,"jill_obf.tim"},
	{4728,76,"epif0.tim"},
	{4804,76,"epif1.tim"},
	{4880,76,"epif2.tim"},
	{4956,76,"epif3.tim"},
	{5032,76,"epif4.tim"},
	{5108,76,"epif5.tim"},
	{5184,76,"epif6.tim"},
	{5260,76,"epif7.tim"},
	{5336,76,"endf.tim"},
	{5412,76,"epij0.tim"},
	{5488,223,"start0.tim"},
	{5712,151,"start1.tim"},
	/*{5863,223,"start0.tim"},*/ /*copy*/
	/*{6087,151,"start1.tim"},*/ /*copy*/
	{6238,33,"config0.tim"},
	{6271,76,"config1.tim"},
	/*{6347,76,"config1.tim"},*/ /*copy*/
	{6423,82,"config2.tim"},
	{6505,3,"radar.tim"},
	{6508,76,"res0_bgf.tim"},
	{6584,49,"res0_obf.tim"},
	{6633,76,"res3_bgf.tim"},
	{6709,76,"res4_bgf.tim"},
	{6785,76,"res5_bgf.tim"},
	{6861,76,"sele_bgf.tim"},
	{6937,49,"sele_obf.tim"},
	{6986,199,"staff0.tim"},
	{7185,36,"stmain0f.tim"},
	{7221,35,"stmain1f.tim"},
	{7256,35,"stmain2f.tim"},
	{7291,35,"stmain3f.tim"},
	{7326,5,"xxx.tim"},
	{7331,223,"stmojif.tim"},
	{7607,65,""},
	{7672,223,"texf.tim"},
	{7914,223,"warnf.tim"},
	{16181,49,"room/emd/em10.tim"},
	{16230,71,"room/emd/em10.emd"},
	{16301,33,"room/emd/em54.tim"},
	{16334,36,"room/emd/em54.emd"},
	{16370,33,"room/emd/em59.tim"},
	{16403,223,"room/emd/em59.emd"},
	{17739,49,"room/emd/em1c.tim"},
	{17788,223,"room/emd/em1c.emd"},
	{18565,66,"room/emd/em16.tim"},
	{18631,225,"room/emd/em16.emd"},
	{19714,33,"room/emd/em53.tim"},
	{19747,223,"room/emd/em53.emd"},
	{20673,98,"room/emd/em1f.tim"},
	{20771,68,"room/emd/em1f.emd"},
	{23001,33,"room/emd/em11.tim"},
	{23034,121,"room/emd/em11.emd"},
	{23155,33,"room/emd/em14.tim"},
	{23188,121,"room/emd/em14.emd"},
	{23309,33,"room/emd/em15.tim"},
	{23342,119,"room/emd/em15.emd"},
	{23461,49,"room/emd/em34.tim"},
	{23510,71,"room/emd/em34.emd"},
	{24396,82,"room/emd/em1d.tim"},
	{24478,223,"room/emd/em1d.emd"},
	{25141,33,"room/emd/em20.tim"},
	{25174,223,"room/emd/em20.emd"},
	{26770,49,"room/emd/em23.tim"},
	{26819,149,"room/emd/em23.emd"},
	{26968,33,"room/emd/em28.tim"},
	{27001,149,"room/emd/em28.emd"},
	{28195,17,"room/emd/em2d.tim"},
	{28212,234,"room/emd/em2d.emd"},
	{30671,33,"room/emd/em58.tim"},
	{30704,228,"room/emd/em58.emd"},
	{32309,49,"room/emd/em2f.tim"},
	{32358,120,"room/emd/em2f.emd"},
	{37498,33,"room/emd/em12.tim"},
	{37531,121,"room/emd/em12.emd"},
	{37652,49,"room/emd/em17.tim"},
	{37701,123,"room/emd/em17.emd"},
	{40310,33,"room/emd/em52.tim"},
	{40343,37,"room/emd/em52.emd"},
	{40380,33,"room/emd/em55.tim"},
	{40413,36,"room/emd/em55.emd"},
	{40449,33,"room/emd/em5a.tim"},
	{40482,36,"room/emd/em5a.emd"},
	{40518,33,"room/emd/em5c.tim"},
	{40551,223,"room/emd/em5c.emd"},
	{45527,17,"room/emd/em21.tim"},
	{45544,62,"room/emd/em21.emd"},
	{56181,33,"room/emd/em2c.tim"},
	{56214,233,"room/emd/em2c.emd"},
	{57177,17,"room/emd/em32.tim"},
	{57194,223,"room/emd/em32.emd"},
	{58700,82,"room/emd/em19.tim"},
	{58782,122,"room/emd/em19.emd"},
	{59351,17,"room/emd/em27.tim"},
	{59368,58,"room/emd/em27.emd"},
	{63483,33,"room/emd/em50.tim"},
	{63516,40,"room/emd/em50.emd"},
	{67825,82,"room/emd/em1a.tim"},
	{67907,118,"room/emd/em1a.emd"},
	{69738,66,"room/emd/em33.tim"},
	{69804,223,"room/emd/em33.emd"},
	{71457,33,"room/emd/em51.tim"},
	{71490,225,"room/emd/em51.emd"},
	{72754,33,"room/emd/em5b.tim"},
	{72787,37,"room/emd/em5b.emd"},
	{72824,33,"room/emd/em62.tim"},
	{72857,37,"room/emd/em62.emd"},
	{72894,33,"room/emd/em63.tim"},
	{72927,37,"room/emd/em63.emd"},
	{72964,33,"room/emd/em64.tim"},
	{72997,37,"room/emd/em64.emd"},
	{73034,33,"room/emd/em65.tim"},
	{73067,37,"room/emd/em65.emd"},
	{73104,33,"room/emd/em66.tim"},
	{73137,229,"room/emd/em66.emd"},
	{78396,223,""},
	{81757,49,"room/emd/em3a.tim"},
	{81806,24,"room/emd/em3a.emd"},
	{104080,17,"room/emd/em26.tim"},
	{104097,51,"room/emd/em26.emd"},
	{104320,49,"room/emd/em36.tim"},
	{104369,95,"room/emd/em36.emd"},
	{104464,17,"room/emd/em37.tim"},
	{104481,17,"room/emd/em37.emd"},
	{105214,33,"room/emd/em25.tim"},
	{105247,28,"room/emd/em25.emd"},
	{107868,17,"room/emd/em40.tim"},
	{107885,229,"room/emd/em40.emd"},
	{109852,82,"room/emd/em35.tim"},
	{109934,83,"room/emd/em35.emd"},
	{110034,7,"room/emd/em3b.emd"},
	{113521,33,"room/emd/em5f.tim"},
	{113554,37,"room/emd/em5f.emd"},
	{114560,95,"room/emd08/em10.emd"},
	{114688,223,"room/emd08/em20.emd"},
	{115493,95,"room/emd08/em36.emd"},
	{116626,121,"room/emd08/em11.emd"},
	{116780,75,"room/emd08/em15.emd"},
	{118348,125,"room/emd08/em23.emd"},
	{118613,151,"room/emd08/em28.emd"},
	{120023,49,"room/emd/em13.tim"},
	{120072,93,"room/emd08/em13.emd"},
	{120182,75,"room/emd08/em32.emd"},
	{122936,116,"room/emd08/em16.emd"},
	{128637,98,"room/emd/em1b.tim"},
	{128735,228,"room/emd/em1b.emd"},
	{129972,33,"room/emd/em22.tim"},
	{130005,231,"room/emd08/em22.emd"},
	{131076,106,"room/emd08/em1b.emd"},
	{133286,86,"room/emd08/em1f.emd"},
	{133901,33,"room/emd/em56.tim"},
	{133934,229,"room/emd/em56.emd"},
	{136504,33,"room/emd/em24.tim"},
	{136537,223,"room/emd08/em24.emd"},
	{137916,57,"room/emd/em22.emd"},
	{139671,223,"room/emd/em24.emd"},
	{143801,66,"room/emd/em30.tim"},
	{143867,223,"room/emd/em30.emd"},
	{149490,66,"room/emd/em18.tim"},
	{149556,114,"room/emd/em18.emd"},
	{156841,33,"room/emd/em60.tim"},
	{156874,228,"room/emd/em60.emd"},
	{158880,33,"room/emd/em3f.tim"},
	{158913,9,"room/emd/em3f.emd"},
	{161247,66,"room/emd/em38.tim"},
	{161313,135,"room/emd/em38.emd"},
	{161448,17,"room/emd/em39.tim"},
	{161465,229,"room/emd/em39.emd"},
	{162713,33,"room/emd/em3e.tim"},
	{162746,5,"room/emd/em3e.emd"},
	{170554,33,"room/emd/em61.tim"},
	{170587,35,"room/emd/em61.emd"},
	{170622,33,"room/emd/em67.tim"},
	{170655,223,"room/emd/em67.emd"},
	{170934,171,""},
	{172486,68,"room/emd08/em1a.emd"},
	{173233,58,"room/emd08/em27.emd"},
	{176053,223,"room/emd08/em21.emd"},
	{178329,223,"room/emd08/em34.emd"},
	{180793,230,"room/emd08/em19.emd"},
	{191825,223,"room/emd08/em1d.emd"},
	{196580,72,"room/emd08/em17.emd"},
	{202819,62,"room/emd08/em18.emd"},
	{202930,223,"room/emd/em57.emd"}
};

static const re3_cdraw_t re3_sles_02532[]={
	{202,82,""},
	{284,32,""},
	{316,25,""},
	{341,85,""},
	{426,76,""},
	{502,17,""},
	{519,17,""},
	{536,12,""},
	{548,12,""},
	{560,12,""},
	{572,12,""},
	{584,12,""},
	{596,12,""},
	{608,12,""},
	{620,12,""},
	{632,12,""},
	{644,17,""},
	{661,12,""},
	{673,12,""},
	{685,12,""},
	{697,12,""},
	{709,12,""},
	{721,17,""},
	{738,12,""},
	{750,12,""},
	{762,12,""},
	{774,12,""},
	{786,12,""},
	{798,12,""},
	{810,12,""},
	{822,17,""},
	{839,12,""},
	{851,12,""},
	{863,17,""},
	{880,12,""},
	{892,12,""},
	{904,12,""},
	{916,12,""},
	{928,12,""},
	{940,17,""},
	{957,12,""},
	{969,12,""},
	{981,12,""},
	{993,12,""},
	{1005,12,""},
	{1017,17,""},
	{1034,12,""},
	{1046,12,""},
	{1058,12,""},
	{1070,12,""},
	{1082,12,""},
	{1094,17,""},
	{1111,12,""},
	{1123,12,""},
	{1135,12,""},
	{1147,12,""},
	{1159,12,""},
	{1171,12,""},
	{1183,17,""},
	{1200,12,""},
	{1212,12,""},
	{1224,12,""},
	{1236,12,""},
	{1248,12,""},
	{1260,17,""},
	{1277,12,""},
	{1289,12,""},
	{1301,12,""},
	{1313,12,""},
	{1325,12,""},
	{1337,12,""},
	{1349,17,""},
	{1366,12,""},
	{1378,12,""},
	{1390,12,""},
	{1402,17,""},
	{1419,12,""},
	{1431,12,""},
	{1443,12,""},
	{1455,12,""},
	{1467,12,""},
	{1479,12,""},
	{1491,17,""},
	{1508,12,""},
	{1520,12,""},
	{1532,12,""},
	{1544,17,""},
	{1561,12,""},
	{1573,12,""},
	{1585,12,""},
	{1597,12,""},
	{1609,12,""},
	{1621,12,""},
	{1633,12,""},
	{1645,17,""},
	{1662,12,""},
	{1674,12,""},
	{1686,12,""},
	{1698,12,""},
	{1710,12,""},
	{1722,12,""},
	{1734,12,""},
	{1746,12,""},
	{1758,17,""},
	{1775,12,""},
	{1787,12,""},
	{1799,12,""},
	{1811,12,""},
	{1823,17,""},
	{1840,12,""},
	{1852,12,""},
	{1864,12,""},
	{1876,12,""},
	{1888,12,""},
	{1900,17,""},
	{1917,12,""},
	{1929,12,""},
	{1941,12,""},
	{1953,12,""},
	{1965,12,""},
	{1977,17,""},
	{1994,12,""},
	{2006,12,""},
	{2018,12,""},
	{2030,12,""},
	{2042,12,""},
	{2054,12,""},
	{2066,17,""},
	{2083,12,""},
	{2095,12,""},
	{2107,12,""},
	{2119,12,""},
	{2131,17,""},
	{2148,12,""},
	{2160,12,""},
	{2172,12,""},
	{2184,17,""},
	{2201,12,""},
	{2213,12,""},
	{2225,12,""},
	{2237,12,""},
	{2249,17,""},
	{2266,12,""},
	{2278,12,""},
	{2290,12,""},
	{2302,12,""},
	{2314,17,""},
	{2331,12,""},
	{2343,12,""},
	{2355,12,""},
	{2367,12,""},
	{2379,17,""},
	{2396,12,""},
	{2408,12,""},
	{2420,12,""},
	{2432,17,""},
	{2449,12,""},
	{2461,12,""},
	{2473,12,""},
	{2485,17,""},
	{2502,12,""},
	{2514,12,""},
	{2526,12,""},
	{2538,12,""},
	{2550,17,""},
	{2567,12,""},
	{2579,12,""},
	{2591,12,""},
	{2603,12,""},
	{2615,17,""},
	{2632,12,""},
	{2644,12,""},
	{2656,12,""},
	{2668,12,""},
	{2680,12,""},
	{2692,12,""},
	{2704,12,""},
	{2716,12,""},
	{2728,17,""},
	{2745,12,""},
	{2757,12,""},
	{2769,12,""},
	{2781,17,""},
	{2798,12,""},
	{2810,12,""},
	{2822,12,""},
	{2834,12,""},
	{2846,12,""},
	{2858,12,""},
	{2870,12,""},
	{2882,12,""},
	{2894,12,""},
	{2906,17,""},
	{2923,17,""},
	{2940,65,""},
	{3005,65,""},
	{3070,65,""},
	{3135,65,""},
	{3200,65,""},
	{3265,65,""},
	{3330,65,""},
	{3395,65,""},
	{3460,44,""},
	{3504,5,""},
	{3509,5,""},
	{3514,5,""},
	{3519,5,""},
	{3524,5,""},
	{3529,5,""},
	{3534,5,""},
	{3539,5,""},
	{3544,5,""},
	{3549,5,""},
	{3554,5,""},
	{3559,5,""},
	{3564,5,""},
	{3569,5,""},
	{3574,5,""},
	{3579,5,""},
	{3584,5,""},
	{3589,5,""},
	{3594,5,""},
	{3599,5,""},
	{3604,5,""},
	{3609,5,""},
	{3614,5,""},
	{3619,5,""},
	{3624,5,""},
	{3629,5,""},
	{3634,5,""},
	{3639,5,""},
	{3644,5,""},
	{3649,5,""},
	{3654,5,""},
	{3659,5,""},
	{3664,5,""},
	{3669,5,""},
	{3674,5,""},
	{3679,5,""},
	{3684,5,""},
	{3689,5,""},
	{3694,5,""},
	{3699,5,""},
	{3704,5,""},
	{3709,5,""},
	{3714,5,""},
	{3719,5,""},
	{3724,5,""},
	{3729,5,""},
	{3734,5,""},
	{3739,5,""},
	{3744,5,""},
	{3749,5,""},
	{3754,5,""},
	{3759,5,""},
	{3764,5,""},
	{3769,5,""},
	{3774,5,""},
	{3779,5,""},
	{3784,5,""},
	{3789,5,""},
	{3794,5,""},
	{3799,5,""},
	{3804,5,""},
	{3809,5,""},
	{3814,5,""},
	{3819,5,""},
	{3824,5,""},
	{3829,5,""},
	{3834,5,""},
	{3839,5,""},
	{3844,5,""},
	{3849,5,""},
	{3854,5,""},
	{3859,5,""},
	{3864,5,""},
	{3869,5,""},
	{3874,5,""},
	{3879,5,""},
	{3884,5,""},
	{3889,5,""},
	{3894,5,""},
	{3899,5,""},
	{3904,5,""},
	{3909,5,""},
	{3914,5,""},
	{3919,5,""},
	{3924,5,""},
	{3929,5,""},
	{3934,5,""},
	{3939,5,""},
	{3944,5,""},
	{3949,5,""},
	{3954,5,""},
	{3959,5,""},
	{3964,5,""},
	{3969,5,""},
	{3974,5,""},
	{3979,5,""},
	{3984,5,""},
	{3989,5,""},
	{3994,5,""},
	{3999,5,""},
	{4004,5,""},
	{4009,5,""},
	{4014,5,""},
	{4019,5,""},
	{4024,5,""},
	{4029,5,""},
	{4034,5,""},
	{4039,5,""},
	{4044,5,""},
	{4049,5,""},
	{4054,5,""},
	{4059,5,""},
	{4064,5,""},
	{4069,5,""},
	{4074,5,""},
	{4079,5,""},
	{4084,5,""},
	{4089,5,""},
	{4094,5,""},
	{4099,5,""},
	{4104,5,""},
	{4109,5,""},
	{4114,5,""},
	{4119,5,""},
	{4124,5,""},
	{4129,5,""},
	{4134,5,""},
	{4139,5,""},
	{4144,5,""},
	{4149,5,""},
	{4154,5,""},
	{4159,5,""},
	{4164,5,""},
	{4169,139,""},
	{4308,76,""},
	{4384,231,""},
	{4727,76,""},
	{4803,76,""},
	{4879,76,""},
	{4955,76,""},
	{5031,76,""},
	{5107,76,""},
	{5183,76,""},
	{5259,76,""},
	{5335,76,""},
	{5411,76,""},
	{5487,223,""},
	{5711,151,""},
	{5862,223,""},
	{6086,151,""},
	{6237,33,""},
	{6270,76,""},
	{6346,76,""},
	{6422,82,""},
	{6504,3,""},
	{6507,76,""},
	{6583,49,""},
	{6632,76,""},
	{6708,76,""},
	{6784,76,""},
	{6860,76,""},
	{6936,49,""},
	{6985,199,""},
	{7184,36,""},
	{7220,35,""},
	{7255,35,""},
	{7290,35,""},
	{7325,5,""},
	{7330,223,""},
	{7606,65,""},
	{7671,223,""},
	{7913,223,""},
	{16179,49,"room/emd/em10.tim"},
	{16228,71,"room/emd/em10.emd"},
	{16299,33,"room/emd/em54.tim"},
	{16332,36,"room/emd/em54.emd"},
	{16368,33,"room/emd/em59.tim"},
	{16401,223,"room/emd/em59.emd"},
	{17737,49,"room/emd/em1c.tim"},
	{17786,223,"room/emd/em1c.emd"},
	{18563,66,"room/emd/em16.tim"},
	{18629,225,"room/emd/em16.emd"},
	{19712,33,"room/emd/em53.tim"},
	{19745,223,"room/emd/em53.emd"},
	{20671,98,"room/emd/em1f.tim"},
	{20769,68,"room/emd/em1f.emd"},
	{22999,33,"room/emd/em11.tim"},
	{23032,121,"room/emd/em11.emd"},
	{23153,33,"room/emd/em14.tim"},
	{23186,121,"room/emd/em14.emd"},
	{23307,33,"room/emd/em15.tim"},
	{23340,119,"room/emd/em15.emd"},
	{23459,49,"room/emd/em34.tim"},
	{23508,71,"room/emd/em34.emd"},
	{24394,82,"room/emd/em1d.tim"},
	{24476,223,"room/emd/em1d.emd"},
	{25139,33,"room/emd/em20.tim"},
	{25172,223,"room/emd/em20.emd"},
	{26768,49,"room/emd/em23.tim"},
	{26817,149,"room/emd/em23.emd"},
	{26966,33,"room/emd/em28.tim"},
	{26999,149,"room/emd/em28.emd"},
	{28193,17,"room/emd/em2d.tim"},
	{28210,234,"room/emd/em2d.emd"},
	{30669,33,"room/emd/em58.tim"},
	{30702,228,"room/emd/em58.emd"},
	{32307,49,"room/emd/em2f.tim"},
	{32356,120,"room/emd/em2f.emd"},
	{37496,33,"room/emd/em12.tim"},
	{37529,121,"room/emd/em12.emd"},
	{37650,49,"room/emd/em17.tim"},
	{37699,123,"room/emd/em17.emd"},
	{40308,33,"room/emd/em52.tim"},
	{40341,37,"room/emd/em52.emd"},
	{40378,33,"room/emd/em55.tim"},
	{40411,36,"room/emd/em55.emd"},
	{40447,33,"room/emd/em5a.tim"},
	{40480,36,"room/emd/em5a.emd"},
	{40516,33,"room/emd/em5c.tim"},
	{40549,223,"room/emd/em5c.emd"},
	{45525,17,"room/emd/em21.tim"},
	{45542,62,"room/emd/em21.emd"},
	{56179,33,"room/emd/em2c.tim"},
	{56212,233,"room/emd/em2c.emd"},
	{57175,17,"room/emd/em32.tim"},
	{57192,223,"room/emd/em32.emd"},
	{58698,82,"room/emd/em19.tim"},
	{58780,122,"room/emd/em19.emd"},
	{59349,17,"room/emd/em27.tim"},
	{59366,58,"room/emd/em27.emd"},
	{63480,33,"room/emd/em50.tim"},
	{63513,40,"room/emd/em50.emd"},
	{67822,82,"room/emd/em1a.tim"},
	{67904,118,"room/emd/em1a.emd"},
	{69735,66,"room/emd/em33.tim"},
	{69801,223,"room/emd/em33.emd"},
	{71454,33,"room/emd/em51.tim"},
	{71487,225,"room/emd/em51.emd"},
	{72751,33,"room/emd/em5b.tim"},
	{72784,37,"room/emd/em5b.emd"},
	{72821,33,"room/emd/em62.tim"},
	{72854,37,"room/emd/em62.emd"},
	{72891,33,"room/emd/em63.tim"},
	{72924,37,"room/emd/em63.emd"},
	{72961,33,"room/emd/em64.tim"},
	{72994,37,"room/emd/em64.emd"},
	{73031,33,"room/emd/em65.tim"},
	{73064,37,"room/emd/em65.emd"},
	{73101,33,"room/emd/em66.tim"},
	{73134,229,"room/emd/em66.emd"},
	{81754,49,"room/emd/em3a.tim"},
	{81803,24,"room/emd/em3a.emd"},
	{104076,17,"room/emd/em26.tim"},
	{104093,51,"room/emd/em26.emd"},
	{104316,49,"room/emd/em36.tim"},
	{104365,95,"room/emd/em36.emd"},
	{104460,17,"room/emd/em37.tim"},
	{104477,17,"room/emd/em37.emd"},
	{105210,33,"room/emd/em25.tim"},
	{105243,28,"room/emd/em25.emd"},
	{107630,17,"room/emd/em40.tim"},
	{107647,229,"room/emd/em40.emd"},
	{109614,82,"room/emd/em35.tim"},
	{109696,83,"room/emd/em35.emd"},
	{109796,7,"room/emd/em3b.emd"},
	{113283,33,"room/emd/em5f.tim"},
	{113316,37,"room/emd/em5f.emd"},
	{114322,95,"room/emd08/em10.emd"},
	{114450,223,"room/emd08/em20.emd"},
	{115255,95,"room/emd08/em36.emd"},
	{116388,121,"room/emd08/em11.emd"},
	{116542,75,"room/emd08/em15.emd"},
	{118110,125,"room/emd08/em23.emd"},
	{118375,151,"room/emd08/em28.emd"},
	{119785,49,"room/emd/em13.tim"},
	{119834,93,"room/emd08/em13.emd"},
	{119944,75,"room/emd08/em32.emd"},
	{122698,116,"room/emd08/em16.emd"},
	{128399,98,"room/emd/em1b.tim"},
	{128497,228,"room/emd/em1b.emd"},
	{129732,33,"room/emd/em22.tim"},
	{129765,231,"room/emd08/em22.emd"},
	{130836,106,"room/emd08/em1b.emd"},
	{133046,86,"room/emd08/em1f.emd"},
	{133661,33,"room/emd/em56.tim"},
	{133694,229,"room/emd/em56.emd"},
	{136264,33,"room/emd/em24.tim"},
	{136297,223,"room/emd08/em24.emd"},
	{137676,57,"room/emd/em22.emd"},
	{139431,223,"room/emd/em24.emd"},
	{143385,66,"room/emd/em30.tim"},
	{143451,223,"room/emd/em30.emd"},
	{149074,66,"room/emd/em18.tim"},
	{149140,114,"room/emd/em18.emd"},
	{156424,33,"room/emd/em60.tim"},
	{156457,228,"room/emd/em60.emd"},
	{158463,33,"room/emd/em3f.tim"},
	{158496,9,"room/emd/em3f.emd"},
	{160830,66,"room/emd/em38.tim"},
	{160896,135,"room/emd/em38.emd"},
	{161031,17,"room/emd/em39.tim"},
	{161048,229,"room/emd/em39.emd"},
	{162296,33,"room/emd/em3e.tim"},
	{162329,5,"room/emd/em3e.emd"},
	{170137,33,"room/emd/em61.tim"},
	{170170,35,"room/emd/em61.emd"},
	{170205,33,"room/emd/em67.tim"},
	{170238,223,"room/emd/em67.emd"},
	{170517,171,""},
	{172069,68,"room/emd08/em1a.emd"},
	{172816,58,"room/emd08/em27.emd"},
	{175636,223,"room/emd08/em21.emd"},
	{177912,223,"room/emd08/em34.emd"},
	{180376,230,"room/emd08/em19.emd"},
	{191408,223,"room/emd08/em1d.emd"},
	{196163,72,"room/emd08/em17.emd"},
	{202402,62,"room/emd08/em18.emd"},
	{202513,223,"room/emd/em57.emd"}
};

static const re3_cdraw_t re3_slus_00923[]={
	{199,76,""},
	{275,82,""},
	{357,32,""},
	{389,34,""},
	{423,76,""},
	{499,17,""},
	{516,76,""},
	{592,17,""},
	{609,17,""},
	{626,12,""},
	{638,12,""},
	{650,12,""},
	{662,12,""},
	{674,12,""},
	{686,17,""},
	{703,12,""},
	{715,12,""},
	{727,12,""},
	{739,17,""},
	{756,12,""},
	{768,12,""},
	{780,12,""},
	{792,12,""},
	{804,17,""},
	{821,12,""},
	{833,12,""},
	{845,17,""},
	{862,12,""},
	{874,12,""},
	{886,12,""},
	{898,17,""},
	{915,12,""},
	{927,12,""},
	{939,12,""},
	{951,12,""},
	{963,17,""},
	{980,12,""},
	{992,12,""},
	{1004,12,""},
	{1016,12,""},
	{1028,17,""},
	{1045,12,""},
	{1057,12,""},
	{1069,12,""},
	{1081,12,""},
	{1093,17,""},
	{1110,12,""},
	{1122,12,""},
	{1134,12,""},
	{1146,17,""},
	{1163,12,""},
	{1175,12,""},
	{1187,12,""},
	{1199,12,""},
	{1211,17,""},
	{1228,12,""},
	{1240,12,""},
	{1252,12,""},
	{1264,12,""},
	{1276,17,""},
	{1293,12,""},
	{1305,12,""},
	{1317,12,""},
	{1329,12,""},
	{1341,17,""},
	{1358,12,""},
	{1370,12,""},
	{1382,17,""},
	{1399,12,""},
	{1411,12,""},
	{1423,12,""},
	{1435,12,""},
	{1447,17,""},
	{1464,12,""},
	{1476,12,""},
	{1488,12,""},
	{1500,12,""},
	{1512,17,""},
	{1529,12,""},
	{1541,12,""},
	{1553,12,""},
	{1565,17,""},
	{1582,12,""},
	{1594,12,""},
	{1606,12,""},
	{1618,17,""},
	{1635,12,""},
	{1647,12,""},
	{1659,12,""},
	{1671,12,""},
	{1683,17,""},
	{1700,12,""},
	{1712,12,""},
	{1724,12,""},
	{1736,12,""},
	{1748,17,""},
	{1765,12,""},
	{1777,12,""},
	{1789,12,""},
	{1801,17,""},
	{1818,12,""},
	{1830,12,""},
	{1842,17,""},
	{1859,12,""},
	{1871,12,""},
	{1883,12,""},
	{1895,17,""},
	{1912,12,""},
	{1924,12,""},
	{1936,12,""},
	{1948,12,""},
	{1960,17,""},
	{1977,12,""},
	{1989,12,""},
	{2001,12,""},
	{2013,12,""},
	{2025,17,""},
	{2042,12,""},
	{2054,12,""},
	{2066,17,""},
	{2083,12,""},
	{2095,12,""},
	{2107,12,""},
	{2119,17,""},
	{2136,12,""},
	{2148,12,""},
	{2160,12,""},
	{2172,12,""},
	{2184,17,""},
	{2201,12,""},
	{2213,12,""},
	{2225,12,""},
	{2237,12,""},
	{2249,17,""},
	{2266,12,""},
	{2278,12,""},
	{2290,12,""},
	{2302,12,""},
	{2314,12,""},
	{2326,17,""},
	{2343,12,""},
	{2355,12,""},
	{2367,12,""},
	{2379,17,""},
	{2396,12,""},
	{2408,12,""},
	{2420,12,""},
	{2432,12,""},
	{2444,12,""},
	{2456,17,""},
	{2473,12,""},
	{2485,12,""},
	{2497,12,""},
	{2509,12,""},
	{2521,12,""},
	{2533,12,""},
	{2545,12,""},
	{2557,12,""},
	{2569,12,""},
	{2581,17,""},
	{2598,12,""},
	{2610,12,""},
	{2622,12,""},
	{2634,12,""},
	{2646,12,""},
	{2658,17,""},
	{2675,12,""},
	{2687,12,""},
	{2699,12,""},
	{2711,12,""},
	{2723,12,""},
	{2735,12,""},
	{2747,12,""},
	{2759,17,""},
	{2776,12,""},
	{2788,12,""},
	{2800,17,""},
	{2817,12,""},
	{2829,12,""},
	{2841,12,""},
	{2853,12,""},
	{2865,12,""},
	{2877,17,""},
	{2894,12,""},
	{2906,12,""},
	{2918,12,""},
	{2930,12,""},
	{2942,12,""},
	{2954,17,""},
	{2971,12,""},
	{2983,12,""},
	{2995,12,""},
	{3007,12,""},
	{3019,12,""},
	{3031,17,""},
	{3048,12,""},
	{3060,12,""},
	{3072,12,""},
	{3084,12,""},
	{3096,12,""},
	{3108,12,""},
	{3120,17,""},
	{3137,12,""},
	{3149,12,""},
	{3161,12,""},
	{3173,12,""},
	{3185,12,""},
	{3197,17,""},
	{3214,12,""},
	{3226,12,""},
	{3238,12,""},
	{3250,12,""},
	{3262,12,""},
	{3274,12,""},
	{3286,17,""},
	{3303,12,""},
	{3315,12,""},
	{3327,12,""},
	{3339,17,""},
	{3356,12,""},
	{3368,12,""},
	{3380,12,""},
	{3392,12,""},
	{3404,12,""},
	{3416,12,""},
	{3428,17,""},
	{3445,12,""},
	{3457,12,""},
	{3469,12,""},
	{3481,17,""},
	{3498,12,""},
	{3510,12,""},
	{3522,12,""},
	{3534,12,""},
	{3546,12,""},
	{3558,12,""},
	{3570,17,""},
	{3587,12,""},
	{3599,12,""},
	{3611,12,""},
	{3623,12,""},
	{3635,12,""},
	{3647,12,""},
	{3659,12,""},
	{3671,12,""},
	{3683,17,""},
	{3700,12,""},
	{3712,12,""},
	{3724,12,""},
	{3736,12,""},
	{3748,17,""},
	{3765,12,""},
	{3777,12,""},
	{3789,12,""},
	{3801,12,""},
	{3813,12,""},
	{3825,17,""},
	{3842,12,""},
	{3854,12,""},
	{3866,12,""},
	{3878,12,""},
	{3890,12,""},
	{3902,12,""},
	{3914,17,""},
	{3931,12,""},
	{3943,12,""},
	{3955,12,""},
	{3967,12,""},
	{3979,12,""},
	{3991,12,""},
	{4003,17,""},
	{4020,12,""},
	{4032,12,""},
	{4044,12,""},
	{4056,12,""},
	{4068,17,""},
	{4085,12,""},
	{4097,12,""},
	{4109,17,""},
	{4126,12,""},
	{4138,12,""},
	{4150,12,""},
	{4162,12,""},
	{4174,17,""},
	{4191,12,""},
	{4203,12,""},
	{4215,12,""},
	{4227,12,""},
	{4239,17,""},
	{4256,12,""},
	{4268,12,""},
	{4280,12,""},
	{4292,12,""},
	{4304,17,""},
	{4321,12,""},
	{4333,12,""},
	{4345,17,""},
	{4362,12,""},
	{4374,12,""},
	{4386,12,""},
	{4398,17,""},
	{4415,12,""},
	{4427,12,""},
	{4439,12,""},
	{4451,12,""},
	{4463,17,""},
	{4480,12,""},
	{4492,12,""},
	{4504,12,""},
	{4516,12,""},
	{4528,17,""},
	{4545,12,""},
	{4557,12,""},
	{4569,12,""},
	{4581,12,""},
	{4593,12,""},
	{4605,12,""},
	{4617,12,""},
	{4629,12,""},
	{4641,17,""},
	{4658,12,""},
	{4670,12,""},
	{4682,12,""},
	{4694,17,""},
	{4711,12,""},
	{4723,12,""},
	{4735,12,""},
	{4747,12,""},
	{4759,12,""},
	{4771,12,""},
	{4783,12,""},
	{4795,12,""},
	{4807,17,""},
	{4824,17,""},
	{4841,17,""},
	{4858,17,""},
	{4875,65,""},
	{4940,17,""},
	{4957,65,""},
	{5022,17,""},
	{5039,65,""},
	{5104,17,""},
	{5121,65,""},
	{5186,17,""},
	{5203,65,""},
	{5268,17,""},
	{5285,65,""},
	{5350,17,""},
	{5367,65,""},
	{5432,43,""},
	{5475,5,""},
	{5480,5,""},
	{5485,5,""},
	{5490,5,""},
	{5495,5,""},
	{5500,5,""},
	{5505,5,""},
	{5510,5,""},
	{5515,5,""},
	{5520,5,""},
	{5525,5,""},
	{5530,5,""},
	{5535,5,""},
	{5540,5,""},
	{5545,5,""},
	{5550,5,""},
	{5555,5,""},
	{5560,5,""},
	{5565,5,""},
	{5570,5,""},
	{5575,5,""},
	{5580,5,""},
	{5585,5,""},
	{5590,5,""},
	{5595,5,""},
	{5600,5,""},
	{5605,5,""},
	{5610,5,""},
	{5615,5,""},
	{5620,5,""},
	{5625,5,""},
	{5630,5,""},
	{5635,5,""},
	{5640,5,""},
	{5645,5,""},
	{5650,5,""},
	{5655,5,""},
	{5660,5,""},
	{5665,5,""},
	{5670,5,""},
	{5675,5,""},
	{5680,5,""},
	{5685,5,""},
	{5690,5,""},
	{5695,5,""},
	{5700,5,""},
	{5705,5,""},
	{5710,5,""},
	{5715,5,""},
	{5720,5,""},
	{5725,5,""},
	{5730,5,""},
	{5735,5,""},
	{5740,5,""},
	{5745,5,""},
	{5750,5,""},
	{5755,5,""},
	{5760,5,""},
	{5765,5,""},
	{5770,5,""},
	{5775,5,""},
	{5780,5,""},
	{5785,5,""},
	{5790,5,""},
	{5795,5,""},
	{5800,5,""},
	{5805,5,""},
	{5810,5,""},
	{5815,5,""},
	{5820,5,""},
	{5825,5,""},
	{5830,5,""},
	{5835,5,""},
	{5840,5,""},
	{5845,5,""},
	{5850,5,""},
	{5855,5,""},
	{5860,5,""},
	{5865,5,""},
	{5870,5,""},
	{5875,5,""},
	{5880,5,""},
	{5885,5,""},
	{5890,5,""},
	{5895,5,""},
	{5900,5,""},
	{5905,5,""},
	{5910,5,""},
	{5915,5,""},
	{5920,5,""},
	{5925,5,""},
	{5930,5,""},
	{5935,5,""},
	{5940,5,""},
	{5945,5,""},
	{5950,5,""},
	{5955,5,""},
	{5960,5,""},
	{5965,5,""},
	{5970,5,""},
	{5975,5,""},
	{5980,5,""},
	{5985,5,""},
	{5990,5,""},
	{5995,5,""},
	{6000,5,""},
	{6005,5,""},
	{6010,5,""},
	{6015,5,""},
	{6020,5,""},
	{6025,5,""},
	{6030,5,""},
	{6035,5,""},
	{6040,5,""},
	{6045,5,""},
	{6050,5,""},
	{6055,5,""},
	{6060,5,""},
	{6065,5,""},
	{6070,5,""},
	{6075,5,""},
	{6080,5,""},
	{6085,5,""},
	{6090,5,""},
	{6095,5,""},
	{6100,5,""},
	{6105,5,""},
	{6110,5,""},
	{6115,5,""},
	{6120,5,""},
	{6125,5,""},
	{6130,5,""},
	{6135,5,""},
	{6140,139,""},
	{6279,76,""},
	{6355,76,""},
	{6431,33,""},
	{6464,231,""},
	{7117,76,""},
	{7193,76,""},
	{7269,76,""},
	{7345,76,""},
	{7421,76,""},
	{7497,76,""},
	{7573,76,""},
	{7649,76,""},
	{7725,76,""},
	{7801,76,""},
	{7877,76,""},
	{7953,76,""},
	{8029,76,""},
	{8105,76,""},
	{8181,76,""},
	{8257,76,""},
	{8333,76,""},
	{8409,76,""},
	{8485,76,""},
	{8561,76,""},
	{8637,223,""},
	{8861,151,""},
	{9012,33,""},
	{9045,76,""},
	{9121,76,""},
	{9197,76,""},
	{9273,33,""},
	{9306,76,""},
	{9382,76,""},
	{9458,82,""},
	{9540,3,""},
	{9543,76,""},
	{9619,76,""},
	{9695,49,""},
	{9744,49,""},
	{9793,76,""},
	{9869,76,""},
	{9945,76,""},
	{10021,76,""},
	{10097,76,""},
	{10173,76,""},
	{10249,76,""},
	{10325,76,""},
	{10401,76,""},
	{10477,76,""},
	{10553,49,""},
	{10602,49,""},
	{10651,199,""},
	{10850,199,""},
	{11049,36,""},
	{11085,36,""},
	{11121,35,""},
	{11156,35,""},
	{11191,35,""},
	{11226,35,""},
	{11261,35,""},
	{11296,35,""},
	{11331,5,""},
	{11336,5,""},
	{11341,223,""},
	{11617,65,""},
	{11682,65,""},
	{11747,167,""},
	{11914,183,""},
	{12097,167,""},
	{12264,223,""},
	{12506,76,""},
	{12582,76,""},
	{12658,223,""},
	{20769,49,"room/emd/em10.tim"},
	{20818,71,"room/emd/em10.emd"},
	{20889,33,"room/emd/em54.tim"},
	{20922,36,"room/emd/em54.emd"},
	{20958,33,"room/emd/em59.tim"},
	{20991,223,"room/emd/em59.emd"},
	{22327,49,"room/emd/em1c.tim"},
	{22376,223,"room/emd/em1c.emd"},
	{23153,66,"room/emd/em16.tim"},
	{23219,225,"room/emd/em16.emd"},
	{24302,33,"room/emd/em53.tim"},
	{24335,223,"room/emd/em53.emd"},
	{25261,98,"room/emd/em1f.tim"},
	{25359,68,"room/emd/em1f.emd"},
	{27589,33,"room/emd/em11.tim"},
	{27622,121,"room/emd/em11.emd"},
	{27743,33,"room/emd/em14.tim"},
	{27776,121,"room/emd/em14.emd"},
	{27897,33,"room/emd/em15.tim"},
	{27930,119,"room/emd/em15.emd"},
	{28049,49,"room/emd/em34.tim"},
	{28098,71,"room/emd/em34.emd"},
	{28984,82,"room/emd/em1d.tim"},
	{29066,223,"room/emd/em1d.emd"},
	{29729,33,"room/emd/em20.tim"},
	{29762,223,"room/emd/em20.emd"},
	{31358,49,"room/emd/em23.tim"},
	{31407,149,"room/emd/em23.emd"},
	{31556,33,"room/emd/em28.tim"},
	{31589,149,"room/emd/em28.emd"},
	{32783,17,"room/emd/em2d.tim"},
	{32800,234,"room/emd/em2d.emd"},
	{34886,33,"room/emd/em58.tim"},
	{34919,228,"room/emd/em58.emd"},
	{36524,49,"room/emd/em2f.tim"},
	{36573,120,"room/emd/em2f.emd"},
	{41713,33,"room/emd/em12.tim"},
	{41746,121,"room/emd/em12.emd"},
	{41867,49,"room/emd/em17.tim"},
	{41916,123,"room/emd/em17.emd"},
	{44525,33,"room/emd/em52.tim"},
	{44558,37,"room/emd/em52.emd"},
	{44595,33,"room/emd/em2e.tim"},
	{44628,36,"room/emd/em55.emd"},
	{44664,33,"room/emd/em5a.tim"},
	{44697,36,"room/emd/em5a.emd"},
	{44733,33,"room/emd/em5c.tim"},
	{44766,223,"room/emd/em5c.emd"},
	{49742,17,"room/emd/em21.tim"},
	{49759,62,"room/emd/em21.emd"},
	{60359,33,"room/emd/em2c.tim"},
	{60392,233,"room/emd/em2c.emd"},
	{61355,17,"room/emd/em32.tim"},
	{61372,223,"room/emd/em32.emd"},
	{62878,82,"room/emd/em19.tim"},
	{62960,122,"room/emd/em19.emd"},
	{63529,17,"room/emd/em27.tim"},
	{63546,58,"room/emd/em27.emd"},
	{67660,33,"room/emd/em50.tim"},
	{67693,40,"room/emd/em50.emd"},
	{72002,82,"room/emd/em1a.tim"},
	{72084,118,"room/emd/em1a.emd"},
	{73915,66,"room/emd/em33.tim"},
	{73981,223,"room/emd/em33.emd"},
	{75634,33,"room/emd/em51.tim"},
	{75667,225,"room/emd/em51.emd"},
	{76931,33,"room/emd/em5b.tim"},
	{76964,37,"room/emd/em5b.emd"},
	{77001,33,"room/emd/em62.tim"},
	{77034,37,"room/emd/em62.emd"},
	{77071,33,"room/emd/em63.tim"},
	{77104,37,"room/emd/em63.emd"},
	{77141,33,"room/emd/em64.tim"},
	{77174,37,"room/emd/em64.emd"},
	{77211,33,"room/emd/em65.tim"},
	{77244,37,"room/emd/em65.emd"},
	{77281,33,"room/emd/em66.tim"},
	{77314,229,"room/emd/em66.emd"},
	{85934,49,"room/emd/em3a.tim"},
	{85983,24,"room/emd/em3a.emd"},
	{108196,17,"room/emd/em26.tim"},
	{108213,51,"room/emd/em26.emd"},
	{108436,49,"room/emd/em36.tim"},
	{108485,95,"room/emd/em36.emd"},
	{108580,17,"room/emd/em37.tim"},
	{108597,17,"room/emd/em37.emd"},
	{109330,33,"room/emd/em25.tim"},
	{109363,28,"room/emd/em25.emd"},
	{111750,17,"room/emd/em40.tim"},
	{111767,229,"room/emd/em40.emd"},
	{113734,82,"room/emd/em35.tim"},
	{113816,83,"room/emd/em35.emd"},
	{113916,7,"room/emd/em3b.emd"},
	{117403,33,"room/emd/em5f.tim"},
	{117436,37,"room/emd/em5f.emd"},
	{118442,95,"room/emd08/em10.emd"},
	{118570,223,"room/emd08/em20.emd"},
	{119375,95,"room/emd08/em36.emd"},
	{120508,121,"room/emd08/em11.emd"},
	{120662,75,"room/emd08/em15.emd"},
	{122230,125,"room/emd08/em23.emd"},
	{122495,151,"room/emd08/em28.emd"},
	{123905,49,"room/emd/em13.tim"},
	{123954,93,"room/emd08/em13.emd"},
	{124064,75,"room/emd08/em32.emd"},
	{126818,116,"room/emd08/em16.emd"},
	{132461,98,"room/emd/em1b.tim"},
	{132559,228,"room/emd/em1b.emd"},
	{133637,33,"room/emd/em22.tim"},
	{133670,231,"room/emd08/em22.emd"},
	{134741,106,"room/emd08/em1b.emd"},
	{136951,86,"room/emd08/em1f.emd"},
	{137566,33,"room/emd/em56.tim"},
	{137599,229,"room/emd/em56.emd"},
	{140169,33,"room/emd/em24.tim"},
	{140202,223,"room/emd08/em24.emd"},
	{141581,57,"room/emd/em22.emd"},
	{143336,223,"room/emd/em24.emd"},
	{147290,66,"room/emd/em30.tim"},
	{147356,223,"room/emd/em30.emd"},
	{152921,66,"room/emd/em18.tim"},
	{152987,114,"room/emd/em18.emd"},
	{160270,33,"room/emd/em60.tim"},
	{160303,228,"room/emd/em60.emd"},
	{162308,33,"room/emd/em3f.tim"},
	{162341,9,"room/emd/em3f.emd"},
	{164675,66,"room/emd/em38.tim"},
	{164741,135,"room/emd/em38.emd"},
	{164876,17,"room/emd/em39.tim"},
	{164893,229,"room/emd/em39.emd"},
	{166141,33,"room/emd/em3e.tim"},
	{166174,5,"room/emd/em3e.emd"},
	{173924,33,"room/emd/em61.tim"},
	{173957,35,"room/emd/em61.emd"},
	{173992,33,"room/emd/em67.tim"},
	{174025,223,"room/emd/em67.emd"},
	{174304,171,""},
	{175856,68,"room/emd08/em1a.emd"},
	{176603,58,"room/emd08/em27.emd"},
	{179423,223,"room/emd08/em21.emd"},
	{181699,223,"room/emd08/em34.emd"},
	{184163,230,"room/emd08/em19.emd"},
	{195137,223,"room/emd08/em1d.emd"},
	{199891,72,"room/emd08/em17.emd"},
	{206130,62,"room/emd08/em18.emd"},
	{206241,223,"room/emd/em57.emd"}
};

/*--- Variables ---*/

static int game_lang = 'u';

/*--- Functions prototypes ---*/

static void re3ps1game_shutdown(void);

static void re3ps1game_loadbackground(void);

static void re3ps1game_loadroom(void);
static int re3ps1game_loadroom_ard(const char *filename);

static void load_font(void);
static void get_char(int ascii, int *x, int *y, int *w, int *h);

/*--- Functions ---*/

void re3ps1game_init(state_t *game_state)
{
	game_state->priv_load_background = re3ps1game_loadbackground;
	game_state->priv_load_room = re3ps1game_loadroom;
	game_state->priv_shutdown = re3ps1game_shutdown;

	game_state->movies_list = (char **) re3ps1game_movies;

	if (state_game_file_exists("cd_data/etc/sele_obf.tim")) {
		game_lang = 'f';
	}

	game_state->load_font = load_font;
	game_state->get_char = get_char;
}

static void re3ps1game_shutdown(void)
{
}

static void re3ps1game_loadbackground(void)
{
	char *filepath;

	filepath = malloc(strlen(re3ps1game_bg)+8);
	if (!filepath) {
		fprintf(stderr, "Can not allocate mem for filepath\n");
		return;
	}
	sprintf(filepath, re3ps1game_bg, game_state.num_stage, game_state.num_stage, game_state.num_room);

	logMsg(1, "bss: Start loading %s ...\n", filepath);

	logMsg(1, "bss: %s loading %s ...\n",
		background_bss_load(filepath, CHUNK_SIZE, 0) ? "Done" : "Failed",
		filepath);

	free(filepath);
}

static void re3ps1game_loadroom(void)
{
	char *filepath;

	filepath = malloc(strlen(re3ps1game_room)+8);
	if (!filepath) {
		fprintf(stderr, "Can not allocate mem for filepath\n");
		return;
	}
	sprintf(filepath, re3ps1game_room, game_state.num_stage, game_state.num_stage, game_state.num_room);

	logMsg(1, "ard: Start loading %s ...\n", filepath);

	logMsg(1, "ard: %s loading %s ...\n",
		re3ps1game_loadroom_ard(filepath) ? "Done" : "Failed",
		filepath);

	free(filepath);
}

static int re3ps1game_loadroom_ard(const char *filename)
{
	PHYSFS_sint64 length;
	Uint8 *ard_file;
	ard_object_t *ard_object;
	int i, count;
	Uint32 offset, len;
	void *file;

	ard_file = (Uint8 *) FS_Load(filename, &length);
	if (!ard_file) {
		return 0;
	}

	count = ((ard_header_t *) ard_file)->count;
	count = SDL_SwapLE32(count);
/*
	offset = 0x800;
	len = 0;
	ard_object = (ard_object_t *) (&ard_file[8]);
	for (i=0; i<count; i++) {
		logMsg(1, "ard: object %d at offset 0x%08x\n", i,offset);
		len = SDL_SwapLE32(ard_object->length);

		offset += len;
		offset |= 0x7ff;
		offset ++;
		ard_object++;
	}
*/
	offset = 0x800;
	len = 0;
	ard_object = (ard_object_t *) (&ard_file[8]);
	for (i=0; i<count; i++) {
		len = SDL_SwapLE32(ard_object->length);
		if (i==8) {
			/* Stop on embedded RDT file */
			break;
		}
		offset += len;
		offset |= 0x7ff;
		offset ++;
		ard_object++;
	}

	file = malloc(len);
	if (!file) {
		free(ard_file);
		return 0;
	}

	logMsg(3, "ard: Loading embedded RDT file from offset 0x%08x\n", offset);
	memcpy(file, &ard_file[offset], len);

	game_state.room = room_create(file, len);
	if (!game_state.room) {
		free(file);
		free(ard_file);
		return 0;
	}

	room_rdt2_init(game_state.room);

	free(ard_file);
	return 1;
}

static void load_font(void)
{
	Uint8 *font_file;
	PHYSFS_sint64 length;
	int retval = 0;
	char *filepath;
	const char *filename = re3ps1game_font;

	filepath = malloc(strlen(filename)+8);
	if (!filepath) {
		fprintf(stderr, "Can not allocate mem for filepath\n");
		return;
	}
	sprintf(filepath, filename, game_lang);

	logMsg(1, "Loading font from %s...\n", filepath);

	font_file = FS_Load(filepath, &length);
	if (font_file) {
		game_state.font = render.createTexture(0);
		if (game_state.font) {
			game_state.font->load_from_tim(game_state.font, font_file);
			retval = 1;
		}

		free(font_file);
	}

	logMsg(1, "Loading font from %s... %s\n", filepath, retval ? "Done" : "Failed");

	free(filepath);
}

static void get_char(int ascii, int *x, int *y, int *w, int *h)
{
	*x = *y = 0;
	*w = 8;
	*h = 10;

	if ((ascii<=32) || (ascii>=96+27)) {
		return;
	}

	ascii -= 32;
	*x = 128+ ((ascii & 15)<<3);
	*y = 176+ ((ascii>>4)*10);
}
